@@ -1,4 +1,5 @@
 from bond import *
+import os
 import pkg_resources
 import re
 
@@ -21,13 +22,14 @@ def _strip_newlines(code):
 class PHP(Bond):
     LANG = 'PHP'
 
-    def __init__(self, php="php", args="-a", xargs="", timeout=None, logfile=None):
-        cmd = ' '.join([php, args, xargs])
-        proc = Spawn(cmd, timeout=timeout, logfile=logfile)
+    def __init__(self, cmd="php", args="-a", xargs="", cwd=None, env=os.environ,
+                 trans_except=False, timeout=None, logfile=None):
+        cmd = ' '.join([cmd, args, xargs])
+        proc = Spawn(cmd, cwd=cwd, env=env, timeout=timeout, logfile=logfile)
         try:
             proc.expect(PHP_PROMPT)
         except pexpect.ExceptionPexpect as e:
-            raise BondException('cannot start PHP')
+            raise BondException(self.LANG, 'cannot get an interactive prompt using: ' + cmd)
 
         # inject our prelude
         code = pkg_resources.resource_string(__name__, PHP_PRELUDE)
@@ -37,9 +39,9 @@ class PHP(Bond):
         try:
             proc.expect(r'\r\n{prompt}'.format(prompt=PHP_PROMPT))
         except pexpect.ExceptionPexpect as e:
-            raise BondException('cannot initialize PHP')
+            raise BondException(self.LANG, 'cannot initialize interpreter')
 
         # start the inner repl
-        proc.sendline(r'{PHP_WRAP_PREFIX}_start();'.format(
-            PHP_WRAP_PREFIX=PHP_WRAP_PREFIX))
-        super(PHP, self).__init__(proc)
+        proc.sendline(r'{PHP_WRAP_PREFIX}_start({trans_except});'.format(
+            PHP_WRAP_PREFIX=PHP_WRAP_PREFIX, trans_except=int(trans_except)))
+        super(PHP, self).__init__(proc, trans_except)
