@@ -53,7 +53,6 @@ if(!defined("STDERR"))
 /// Define our own i/o methods
 function __PY_BOND_output($buffer, $phase)
 {
-  global $__PY_BOND_BUFFERS;
   fwrite(STDOUT, $buffer);
 }
 
@@ -89,6 +88,8 @@ function __PY_BOND_get_error()
 
 
 /// Recursive repl
+$__PY_BOND_TRANS_EXCEPT = null;
+
 function __PY_BOND_sendstate($state, $data)
 {
   $enc_ret = json_encode($data);
@@ -100,15 +101,15 @@ function __PY_BOND_sendstate($state, $data)
   __PY_BOND_sendline("$state $enc_ret");
 }
 
-function __PY_BOND_remote($name, $args)
+function __PY_BOND_call($name, $args)
 {
-  __PY_BOND_sendstate("REMOTE", array($name, $args));
+  __PY_BOND_sendstate("CALL", array($name, $args));
   return __PY_BOND_repl();
 }
 
 function __PY_BOND_repl()
 {
-  global $__PY_BOND_BUFFERS;
+  global $__PY_BOND_BUFFERS, $__PY_BOND_TRANS_EXCEPT;
 
   while($line = __PY_BOND_getline())
   {
@@ -129,7 +130,7 @@ function __PY_BOND_repl()
       }
       catch(Exception $e)
       {
-	$err = $e->getMessage();
+	$err = $e;
       }
       break;
 
@@ -142,13 +143,21 @@ function __PY_BOND_repl()
       }
       catch(Exception $e)
       {
-	$err = $e->getMessage();
+	$err = $e;
       }
       break;
 
     case "EXPORT":
-      $code = "function $args() { return __PY_BOND_remote('$args', func_get_args()); }";
-      $ret = eval($code);
+      $name = $args;
+      if(function_exists($name))
+	$err = "Function \"$name\" already exists";
+      else
+      {
+	$code = "function $name() { return __PY_BOND_call('$args', func_get_args()); }";
+	__PY_BOND_clear_error();
+	@eval($code);
+	$err = __PY_BOND_get_error();
+      }
       break;
 
     case "CALL":
@@ -176,13 +185,17 @@ function __PY_BOND_repl()
       }
       catch(Exception $e)
       {
-	$err = $e->getMessage();
+	$err = $e;
       }
       break;
 
     case "RETURN":
       return $args;
 
+    case "EXCEPT":
+    case "ERROR":
+      throw new Exception($args);
+
     default:
       exit(1);
     }
@@ -207,17 +220,22 @@ function __PY_BOND_repl()
     else
     {
       $state = "EXCEPT";
-      $ret = $err;
+      if($__PY_BOND_TRANS_EXCEPT)
+	$ret = $err;
+      else
+	$ret = ($err instanceOf Exception? $err->getMessage(): @"$err");
     }
 
     __PY_BOND_sendstate($state, $ret);
   }
-  exit(0);
+  return 0;
 }
 
-function __PY_BOND_start()
+function __PY_BOND_start($trans_except)
 {
+  global $__PY_BOND_TRANS_EXCEPT;
   ob_start('__PY_BOND_output');
+  $__PY_BOND_TRANS_EXCEPT = (bool)($trans_except);
   __PY_BOND_sendline("READY");
   exit(__PY_BOND_repl());
 }
