@@ -1,4 +1,5 @@
 from bond import *
+import os
 import pkg_resources
 import re
 
@@ -19,13 +20,14 @@ def _strip_newlines(code):
 class Perl(Bond):
     LANG = 'Perl'
 
-    def __init__(self, perl="perl", args="-d -e1", xargs="", timeout=None, logfile=None):
-        cmd = ' '.join([perl, args, xargs])
-        proc = Spawn(cmd, timeout=timeout, logfile=logfile)
+    def __init__(self, cmd="perl", args="-d -e1", xargs="", cwd=None, env=os.environ,
+                 trans_except=False, timeout=None, logfile=None):
+        cmd = ' '.join([cmd, args, xargs])
+        proc = Spawn(cmd, cwd=cwd, env=env, timeout=timeout, logfile=logfile)
         try:
             proc.expect(PERL_PROMPT)
         except pexpect.ExceptionPexpect as e:
-            raise BondException('cannot start Perl')
+            raise BondException(self.LANG, 'cannot get an interactive prompt using: ' + cmd)
 
         # inject our prelude
         code = pkg_resources.resource_string(__name__, PERL_PRELUDE)
@@ -35,9 +37,9 @@ class Perl(Bond):
         try:
             proc.expect(r'\r\n{prompt}'.format(prompt=PERL_PROMPT))
         except pexpect.ExceptionPexpect as e:
-            raise BondException('cannot initialize Perl')
+            raise BondException(self.LANG, 'cannot initialize interpreter')
 
         # start the inner repl
-        proc.sendline(r'{PERL_WRAP_PREFIX}_start();'.format(
-            PERL_WRAP_PREFIX=PERL_WRAP_PREFIX))
-        super(Perl, self).__init__(proc)
+        proc.sendline(r'{PERL_WRAP_PREFIX}_start({trans_except});'.format(
+            PERL_WRAP_PREFIX=PERL_WRAP_PREFIX, trans_except=int(trans_except)))
+        super(Perl, self).__init__(proc, trans_except)
