@@ -58,6 +58,8 @@ sub __PY_BOND_sendline
 
 
 # Recursive repl
+my $__PY_BOND_TRANS_EXCEPT;
+
 sub __PY_BOND_sendstate($$)
 {
   my ($state, $data) = @_;
@@ -70,10 +72,10 @@ sub __PY_BOND_sendstate($$)
   __PY_BOND_sendline("$state $enc_ret");
 }
 
-sub __PY_BOND_remote($$)
+sub __PY_BOND_call($$)
 {
   my ($name, $args) = @_;
-  __PY_BOND_sendstate("REMOTE", [$name, $args]);
+  __PY_BOND_sendstate("CALL", [$name, $args]);
   return __PY_BOND_repl();
 }
 
@@ -107,8 +109,9 @@ sub __PY_BOND_repl()
     }
     elsif($cmd eq "EXPORT")
     {
-      my $code = "sub $args { __PY_BOND_remote('$args', \\\@_); }";
+      my $code = "sub $args { __PY_BOND_call('$args', \\\@_); }";
       $ret = eval($code);
+      $err = $@;
     }
     elsif($cmd eq "CALL")
     {
@@ -130,6 +133,10 @@ sub __PY_BOND_repl()
     {
       return $args;
     }
+    elsif($cmd eq "EXCEPT" or $cmd eq "ERROR")
+    {
+      die($args);
+    }
     else
     {
       exit(1);
@@ -156,16 +163,18 @@ sub __PY_BOND_repl()
     else
     {
       $state = "EXCEPT";
-      $ret = $err;
+      $ret = ($__PY_BOND_TRANS_EXCEPT? $err: "$err");
     }
 
     __PY_BOND_sendstate($state, $ret);
   }
-  exit(0);
+  return 0;
 }
 
-sub __PY_BOND_start()
+sub __PY_BOND_start($)
 {
+  my $trans_except = shift();
+
   *STDIN = IO::Handle->new();
   *STDOUT = $__PY_BOND_BUFFERS{STDOUT};
   *STDERR = $__PY_BOND_BUFFERS{STDERR};
@@ -174,6 +183,7 @@ sub __PY_BOND_start()
     print STDERR shift();
   };
 
+  $__PY_BOND_TRANS_EXCEPT = $trans_except;
   __PY_BOND_sendline("READY");
   exit(__PY_BOND_repl());
 }
