@@ -30,7 +30,7 @@ def __PY_BOND_sendline(line=""):
 
 
 # Serialization methods
-__PY_BOND_PROTOCOL = -1
+__PY_BOND_PROTOCOL = None
 
 def __PY_BOND_dumps(*args):
     return base64.b64encode(cPickle.dumps(args, __PY_BOND_PROTOCOL))
@@ -40,6 +40,8 @@ def __PY_BOND_loads(string):
 
 
 # Recursive repl
+__PY_BOND_TRANS_EXCEPT = None
+
 def __PY_BOND_sendstate(state, data):
     code = None
     try:
@@ -49,12 +51,12 @@ def __PY_BOND_sendstate(state, data):
         code = __PY_BOND_dumps("cannot encode {data}".format(data=str(data)))
     __PY_BOND_sendline("{state} {code}".format(state=state, code=code))
 
-def __PY_BOND_remote(name, args):
-    __PY_BOND_sendstate("REMOTE", [name, args])
+def __PY_BOND_call(name, args):
+    __PY_BOND_sendstate("CALL", [name, args])
     return __PY_BOND_repl()
 
 def __PY_BOND_export(name):
-    globals()[name] = lambda *args: __PY_BOND_remote(name, args)
+    globals()[name] = lambda *args: __PY_BOND_call(name, args)
 
 def __PY_BOND_repl():
     while True:
@@ -88,6 +90,12 @@ def __PY_BOND_repl():
         elif cmd == "RETURN":
             return args
 
+        elif cmd == "EXCEPT":
+            raise args
+
+        elif cmd == "ERROR":
+            raise Exception(args)
+
         else:
             exit(1)
 
@@ -105,18 +113,20 @@ def __PY_BOND_repl():
             state = "RETURN"
         else:
             state = "EXCEPT"
-            ret = err
+            ret = err if __PY_BOND_TRANS_EXCEPT else str(err)
 
         __PY_BOND_sendstate(state, ret)
 
     # stream ended
-    exit(0)
+    return 0
 
 
-def __PY_BOND_start(protocol=-1):
+def __PY_BOND_start(trans_except, protocol):
+    global __PY_BOND_TRANS_EXCEPT, __PY_BOND_PROTOCOL
     sys.stdout = __PY_BOND_BUFFERS['STDOUT']
     sys.stderr = __PY_BOND_BUFFERS['STDERR']
     sys.stdin = open(os.devnull)
+    __PY_BOND_TRANS_EXCEPT = trans_except
     __PY_BOND_PROTOCOL = protocol
     __PY_BOND_sendline("READY")
     exit(__PY_BOND_repl())
