@@ -6,7 +6,7 @@ import sys
 
 class Spawn(pexpect.spawn):
     def __init__(self, *args, **kwargs):
-        kwargs['env'] = {'TERM': 'dumb'}
+        kwargs.setdefault('env', {})['TERM'] = 'dumb'
         super(Spawn, self).__init__(*args, **kwargs)
 
     def sendline(self, *args, **kwargs):
@@ -49,10 +49,11 @@ class RemoteException(BondException):
 class Bond(object):
     LANG = '<unknown>'
 
-    def __init__(self, proc):
+    def __init__(self, proc, trans_except):
         self.channels = {'STDOUT': sys.stdout, 'STDERR': sys.stderr}
         self.bindings = {}
 
+        self.trans_except = trans_except
         self._proc = proc
         try:
             self._proc.expect("READY\r\n")
@@ -64,7 +65,7 @@ class Bond(object):
         return json.loads(string)
 
     def dumps(self, *args):
-        return json.dumps(*args)
+        return json.dumps(*args, skipkeys=False)
 
 
     def _dumps(self, *args):
@@ -83,20 +84,30 @@ class Bond(object):
                 args = self.loads(args)
 
             # interpret the serial protocol
-            if cmd == "OUTPUT":
+            if cmd == "RETURN":
+                return args
+            elif cmd == "OUTPUT":
                 self.channels[args[0]].write(args[1])
                 continue
-            elif cmd == "REMOTE":
-                ret = self.bindings[args[0]](*args[1])
-                ret = self._dumps(ret) if ret else None
-                self._proc.sendline('RETURN {ret}'.format(ret=ret))
-                continue
             elif cmd == "EXCEPT":
                 raise RemoteException(self.LANG, str(args), args)
             elif cmd == "ERROR":
                 raise SerializationException(self.LANG, str(args), 'remote')
-            elif cmd == "RETURN":
-                return args
+            elif cmd == "CALL":
+                ret = None
+                state = "RETURN"
+                try:
+                    ret = self.bindings[args[0]](*args[1])
+                except Exception as e:
+                    state = "EXCEPT"
+                    ret = e if self.trans_except else str(e)
+                try:
+                    ret = self._dumps(ret) if ret else None
+                except SerializationException as e:
+                    state = "ERROR"
+                    ret = self._dumps(str(e))
+                self._proc.sendline('{state} {ret}'.format(state=state, ret=ret))
+                continue
 
             raise BondException(self.LANG, 'unknown interpreter state')
 
