@@ -100,14 +100,14 @@ def test_call_error():
 
     # define a function and call it
     py.eval_block(r'''def test_simple(arg):
-        return 1 / arg
+        return arg
     ''')
-    py.eval('test_simple(1)')
+    assert(py.call('test_simple', 1) == 1)
 
-    # make it fail
+    # unknown function
     failed = False
     try:
-        py.call('test_simple', 0)
+        py.call('test_undefined')
     except bond.RemoteException as e:
         print(e)
         failed = True
@@ -202,6 +202,42 @@ def test_ser_err():
     assert(py.eval('1') == 1)
 
 
+def test_exception():
+    py = Python(timeout=1)
+
+    # remote exception
+    py.eval_block(r'''def exceptional():
+    raise Exception('an exception')
+    ''')
+
+    # ... in eval
+    failed = False
+    try:
+        py.eval('exceptional()')
+    except bond.RemoteException as e:
+        print(e)
+        failed = True
+    assert(failed)
+
+    # ... in eval_block
+    failed = False
+    try:
+        py.eval_block('exceptional()')
+    except bond.RemoteException as e:
+        print(e)
+        failed = True
+    assert(failed)
+
+    # ... in call
+    failed = False
+    try:
+        py.call('exceptional')
+    except bond.RemoteException as e:
+        print(e)
+        failed = True
+    assert(failed)
+
+
 def test_export():
     def call_me():
         return 42
@@ -211,6 +247,37 @@ def test_export():
     assert(py.call('call_me') == 42)
 
 
+def test_export_redef():
+    py = Python(timeout=1)
+
+    def call_me():
+        return 42
+
+    py.export(call_me)
+    try:
+        py.export(call_me)
+    except:
+        pass
+
+    assert(py.call('call_me') == 42)
+
+
+def test_export_invalid():
+    py = Python(timeout=1)
+
+    def call_me():
+        return 42
+
+    try:
+        # Interestingly enough, this works in Python, though it won't be
+        # possible to call the function normally
+        py.export(call_me, 'invalid name')
+    except Exception as e:
+        print(e)
+
+    assert(py.eval('1') == 1)
+
+
 def test_export_recursive():
     py = Python(timeout=1)
 
@@ -273,6 +340,45 @@ def test_export_ser_err():
     assert(py.eval('1') == 1)
 
 
+def test_export_except():
+    py = Python(timeout=1)
+
+    def gen_exception():
+        raise Exception("test")
+
+    py.export(gen_exception)
+    py.eval_block(r'''def test_exception():
+    ret = False
+    try:
+        gen_exception()
+    except Exception as e:
+        ret = (str(e) == "test")
+    return ret
+    ''')
+
+    assert(py.call('test_exception') == True)
+
+
+def test_export_except_ser_err():
+    py = Python(timeout=1)
+
+    def call_me():
+        return lambda x: x
+
+    py.export(call_me)
+    py.eval_block(r'''def test_ser_err():
+    ret = False
+    try:
+        call_me()
+    except Exception as e:
+        print(e)
+        ret = True
+    return ret
+    ''')
+
+    assert(py.call('test_ser_err') == True)
+
+
 def test_output_redirect():
     py = Python(timeout=1)
 
@@ -283,3 +389,68 @@ def test_output_redirect():
     # stderr
     py.eval_block(r'import sys; sys.stderr.write("Hello world!\n");')
     assert(py.eval('1') == 1)
+
+
+def test_trans_except():
+    py_trans = Python(timeout=1, trans_except=True)
+    py_not_trans = Python(timeout=1, trans_except=False)
+
+    code = r'''def func():
+        raise RuntimeError("a runtime error")
+    '''
+
+    # by default exceptions are transparent, so the following should correctly
+    # restore the RuntimeError in RemoteException.data
+    py_trans.eval_block(code)
+    failed = False
+    try:
+        ret = py_trans.call('func')
+    except bond.RemoteException as e:
+        failed = isinstance(e.data, RuntimeError)
+    assert(failed)
+
+    # ensure the env didn't just die
+    assert(py_trans.eval('1') == 1)
+
+    # this one though will just always forward the remote exception
+    py_not_trans.eval_block(code)
+    failed = False
+    try:
+        ret = py_not_trans.call('func')
+    except bond.RemoteException as e:
+        failed = isinstance(e.data, str)
+    assert(failed)
+
+    # ensure the env didn't just die
+    assert(py_not_trans.eval('1') == 1)
+
+
+def test_export_trans_except():
+    py_trans = Python(timeout=1, trans_except=True)
+    py_not_trans = Python(timeout=1, trans_except=False)
+
+    def call_me():
+       raise RuntimeError("a runtime error")
+
+    # by default exceptions are transparent, so the following should correctly
+    # restore the RuntimeError in RemoteException.data
+    py_trans.export(call_me)
+    py_trans.eval_block(r'''if True:
+    failed = False
+    try:
+        call_me()
+    except RuntimeError:
+        failed = True
+    ''')
+    assert(py_trans.eval('failed') == True)
+
+    # this one though will just generate a general Exception
+    py_not_trans.export(call_me)
+    py_not_trans.eval_block(r'''if True:
+    failed = False
+    try:
+        call_me()
+    except Exception as e:
+        failed = not isinstance(e, RuntimeError)
+    ''')
+    assert(py_not_trans.eval('failed') == True)
